@@ -186,7 +186,7 @@ class MaskLoss(nn.Module):
         factors: List[float] = [1],
         f_under: float = 1,
         eps=1e-12,
-        factor=1,
+        factor: float = 1.0,
         gamma_pred: Optional[float] = None,
     ):
         super().__init__()
@@ -196,8 +196,10 @@ class MaskLoss(nn.Module):
             self.mask_fn = irm
         elif mask == "iam":
             self.mask_fn = iam
+        elif mask == "spec":
+            self.mask_fn = None
         else:
-            raise ValueError("Unsupported mask function.")
+            raise ValueError(f"Unsupported mask function: {mask}.")
         self.gamma = gamma
         self.gamma_pred = gamma if gamma_pred is None else gamma_pred
         self.powers = powers
@@ -221,12 +223,20 @@ class MaskLoss(nn.Module):
 
     @torch.jit.export
     def erb_mask_compr(self, clean: Tensor, noisy: Tensor, compressed: bool = True) -> Tensor:
-        mask = self.mask_fn(clean, noisy)
-        mask = torch.matmul(mask, self.erb_fb).clamp_min(self.eps)
+        mask_fn = self.mask_fn or iam
+        mask = mask_fn(clean, noisy)
+        mask = self.erb(mask)
         if compressed:
             mask = mask.pow(self.gamma)
         return mask
 
+    @torch.jit.export
+    def erb(self, x: Tensor, clamp_min=True) -> Tensor:
+        x = torch.matmul(x, self.erb_fb)
+        if clamp_min:
+            x = x.clamp_min(clamp_min)
+        return x
+
     @torch.jit.export
     def erb_inv(self, x: Tensor) -> Tensor:
         return torch.matmul(x, self.erb_inv_fb)
@@ -239,8 +249,12 @@ class MaskLoss(nn.Module):
         if not torch.isfinite(input).all():
             raise ValueError("Input is NaN")
         assert input.min() >= 0
-        g_t = self.erb_mask_compr(clean, noisy, compressed=True)
-        g_p = input.clamp_min(self.eps).pow(self.gamma_pred)
+        if self.mask_fn is not None:
+            g_t = self.erb_mask_compr(clean, noisy, compressed=True)
+            g_p = input.clamp_min(self.eps).pow(self.gamma_pred)
+        else:
+            g_t = self.erb(clean.abs()).pow(self.gamma)  # We use directly the clean spectrum
+            g_p = (self.erb(noisy.abs()) * input).pow(self.gamma_pred)
         loss = torch.zeros((), device=input.device)
         tmp = g_t.sub(g_p).pow(2)
         if self.f_under != 1:
@@ -380,12 +394,18 @@ class LocalSnrLoss(nn.Module):
 
 
 class Loss(nn.Module):
-    ml_f: Final[float]
-    cal_f: Final[float]
-    sl_f: Final[float]
-    mrsl_f: Final[float]
+    """Loss wrapper containing several different loss functions within this file.
+
+    The configuration is done via the config file.
+    """
 
     def __init__(self, state: DF, istft: Optional[Istft] = None):
+        """Loss wrapper containing all methods for loss calculation.
+
+        Args:
+            state (DF): DF state needed for MaskLoss.
+            istft (Callable/Module): Istft method needed for time domain losses.
+        """
         super().__init__()
         p = ModelParams()
         self.lsnr = LocalSnrTarget(ws=20, target_snr_range=[p.lsnr_min - 5, p.lsnr_max + 5])
@@ -397,12 +417,13 @@ class Loss(nn.Module):
         self.summaries: Dict[str, List[Tensor]] = self.reset_summaries()
         # Mask Loss
         self.ml_f = config("factor", 0, float, section="MaskLoss")  # e.g. 1
+        self.ml_mask = config("mask", "iam", str, section="MaskLoss")  # e.g. 1
         self.ml_gamma = config("gamma", 0.6, float, section="MaskLoss")
         self.ml_gamma_pred = config("gamma_pred", 0.6, float, section="MaskLoss")
         self.ml_f_under = config("f_under", 2, float, section="MaskLoss")
         self.ml = MaskLoss(
             state,
-            "iam",
+            mask=self.ml_mask,
             factor=self.ml_f,
             f_under=self.ml_f_under,
             gamma=self.ml_gamma,
@@ -458,6 +479,18 @@ class Loss(nn.Module):
         max_freq: Optional[Tensor] = None,
         multi_stage_specs: List[Tensor] = [],
     ):
+        """Computes all losses.
+
+        Args:
+            clean (Tensor): Clean complex spectrum of shape [B, C, T, F].
+            noisy (Tensor): Noisy complex spectrum of shape [B, C, T, F].
+            enhanced (Tensor): Enhanced complex spectrum of shape [B, C, T, F].
+            mask (Tensor): Mask (real-valued) estimate of shape [B, C, T, E], E: Number of ERB bins.
+            lsnr (Tensor): Local SNR estimates of shape [B, T, 1].
+            snrs (Tensor): Input SNRs of the noisy mixture of shape [B].
+            max_freq (Optional, Tensor): Maximum frequency present in the noisy mixtrue (e.g. due to a lower sampling rate) of shape [B].
+            multi_stage_specs (Optional, Tensor): Enhanced complex spectrums of different intermediate outputs of the DNN.
+        """
         max_bin: Optional[Tensor] = None
         if max_freq is not None:
             max_bin = (
@@ -504,8 +537,7 @@ class Loss(nn.Module):
                 sdrl = self.sdrl(ms, clean_td.expand_as(ms))
             else:
                 sdrl = self.sdrl(enhanced_td, clean_td)
-        if self.store_losses and self.istft is not None:
-            assert enhanced_td is not None
+        if self.store_losses and enhanced_td is not None:
             assert clean_td is not None
             self.store_summaries(
                 enhanced_td,
