@@ -763,10 +763,12 @@ class GroupedLinearEinsum(nn.Module):
 
     def forward(self, x: Tensor) -> Tensor:
         # x: [..., I]
-        new_shape = list(x.shape)[:-1] + [self.groups, self.ws]
+        b, t, _ = x.shape
+        # new_shape = list(x.shape)[:-1] + [self.groups, self.ws]
+        new_shape = (b, t, self.groups, self.ws)
         x = x.view(new_shape)
         # The better way, but not supported by torchscript
-        #     x = x.unflatten(-1, (self.groups, self.ws))  # [..., G, I/G]
+        # x = x.unflatten(-1, (self.groups, self.ws))  # [..., G, I/G]
         x = torch.einsum("btgi,gih->btgh", x, self.weight)  # [..., G, H/G]
         x = x.flatten(2, 3)  # [B, T, H]
         return x
