@@ -221,12 +221,54 @@ class CRM(MultiFrameModule):
 class MfWf(MultiFrameModule):
     """Multi-frame Wiener filter base module."""
 
+    cholesky_decomp: Final[bool]
+    inverse: Final[bool]
+    enforce_constraints: Final[bool]
+    eps: Final[float]
+    dload: Final[float]
+
     def __init__(
-        self, num_freqs: int, frame_size: int, lookahead: int = 0, cholesky_decomp: bool = False
+        self,
+        num_freqs: int,
+        frame_size: int,
+        lookahead: int = 0,
+        cholesky_decomp: bool = False,
+        inverse: bool = True,
+        enforce_constraints: bool = True,
+        eps=1e-8,
+        dload=1e-7,
     ):
-        """Multi-frame Wiener Filter via an estimate of the inverse"""
+        """Multi-frame Wiener Filter via an estimate of the inverse
+
+        Args:
+            num_freqs (int): Number of frequency bins to apply MVDR filtering to.
+            frame_size (int): Frame size of the MF MVDR filter.
+            lookahead (int): Lookahead of the frame.
+            cholesky_decomp (bool): Whether the input is a cholesky decomposition of the correlation matrix. Defauls to `False`.
+            inverse (bool): Whether the input is a normal or inverse correlation matrix. Defaults to `True`.
+            enforce_constraints (bool): Enforce hermetian matrix for non-inverse input and a triangular matrix for cholesky decomposition inpiut.
+        """
         super().__init__(num_freqs, frame_size, lookahead=lookahead)
         self.cholesky_decomp = cholesky_decomp
+        self.inverse = inverse
+        self.enforce_constraints = enforce_constraints
+        self.triu_idcs = torch.triu_indices(self.frame_size, self.frame_size, 1)
+        self.tril_idcs = torch.empty_like(self.triu_idcs)
+        self.tril_idcs[0] = self.triu_idcs[1]
+        self.tril_idcs[1] = self.triu_idcs[0]
+        self.eps = eps
+        self.dload = dload
+
+    def get_r_factor(self):
+        """Return an factor f such that Rxx/f in range [-1, 1]"""
+        if self.inverse and self.cholesky_decomp:
+            return 2e3
+        elif self.inverse and not self.cholesky_decomp:
+            return 3e7
+        elif not self.inverse and self.cholesky_decomp:
+            return 2e-4
+        elif not self.inverse and not self.cholesky_decomp:
+            return 5e-6
 
     def forward(self, spec: Tensor, ifc: Tensor, iRxx: Tensor) -> Tensor:
         """Multi-frame Wiener filter based on Rxx**-1 and speech IFC vector.
@@ -234,19 +276,37 @@ class MfWf(MultiFrameModule):
         Args:
             spec (complex Tensor): Spectrogram of shape [B, 1, T, F]
             ifc (complex Tensor): Inter-frame speech correlation vector [B, T, F, N*2]
-            iRxx (complex Tensor): Inverse noisy covariance matrix Rxx**-1 [B, T, F, (N**2)*2] OR
+            iRxx (complex Tensor): (Inverse) noisy covariance matrix Rxx**-1 [B, T, F, (N**2)*2] OR
                 cholesky_decomp Rxx=L*L^H of the same shape.
 
         Returns:
             spec (complex Tensor): Filtered spectrogram of shape [B, C, T, F]
         """
+
         spec_u = self.spec_unfold(torch.view_as_complex(spec))
-        ifc = torch.view_as_complex(ifc.unflatten(3, (self.frame_size, 2)))
         iRxx = torch.view_as_complex(iRxx.unflatten(3, (self.frame_size, self.frame_size, 2)))
         if self.cholesky_decomp:
-            iRxx = iRxx * iRxx.transpose(3, 4).conj()
+            if self.enforce_constraints:
+                # Upper triangular (wo. diagonal) must be zero
+                iRxx[:, :, :, self.triu_idcs[0], self.triu_idcs[1]] = 0.0
+            # Revert cholesky decomposition
+            iRxx = iRxx.matmul(iRxx.transpose(3, 4).conj())
+        if self.enforce_constraints and not self.inverse and not self.cholesky_decomp:
+            # If we have a cholesky_decomp input the constraints are already enforced.
+            # We have a standard correlation matrix as input. Imaginary part on the diagonal should be 0.
+            torch.diagonal(iRxx, dim1=-1, dim2=-2).imag = 0.0
+            # Triu should be complex conj of tril
+            tril_conj = iRxx[:, :, :, self.tril_idcs[0], self.tril_idcs[1]].conj()
+            iRxx[:, :, :, self.triu_idcs[0], self.triu_idcs[1]] = tril_conj
+        ifc = torch.view_as_complex(ifc.unflatten(3, (self.frame_size, 2)))
         spec_f = spec_u.narrow(-2, 0, self.num_freqs)
-        w = torch.einsum("...nm,...m->...n", iRxx, ifc).unsqueeze(1)  # [B, 1, F, N]
+        if not self.inverse:  # Is already an inverse input. No need to inverse it again.
+            # Regularization on diag for stability
+            iRxx = _tik_reg(iRxx, self.dload, self.eps)
+            # Compute weights by solving the equation system
+            w = torch.linalg.solve(iRxx, ifc).unsqueeze(1)
+        else:  # We already have an inverse estimate. Just compute the linear combination.
+            w = torch.einsum("...nm,...m->...n", iRxx, ifc).unsqueeze(1)  # [B, 1, F, N]
         spec_f = self.apply_coefs(spec_f, w)
         if self.training:
             spec = spec.clone()
@@ -257,24 +317,54 @@ class MfWf(MultiFrameModule):
 class MfMvdr(MultiFrameModule):
     """Multi-frame minimum variance distortionless beamformer based on Rnn**-1 and speech IFC vector."""
 
-    eps: Final[float]
-    normalize_ifc: Final[bool]
     cholesky_decomp: Final[bool]
+    inverse: Final[bool]
+    enforce_constraints: Final[bool]
+    eps: Final[float]
+    dload: Final[float]
 
     def __init__(
         self,
         num_freqs: int,
         frame_size: int,
         lookahead: int = 0,
-        eps: float = 1e-8,
-        normalize_ifc: bool = True,
         cholesky_decomp: bool = False,
+        inverse: bool = True,
+        enforce_constraints: bool = True,
+        eps=1e-8,
+        dload=1e-7,
     ):
-        """Multi-frame minimum variance distortionless beamformer."""
+        """Multi-frame minimum variance distortionless beamformer.
+
+        Args:
+            num_freqs (int): Number of frequency bins to apply MVDR filtering to.
+            frame_size (int): Frame size of the MF MVDR filter.
+            lookahead (int): Lookahead of the frame.
+            cholesky_decomp (bool): Whether the input is a cholesky decomposition of the correlation matrix. Defauls to `False`.
+            inverse (bool): Whether the input is a normal or inverse correlation matrix. Defaults to `True`.
+            enforce_constraints (bool): Enforce hermetian matrix for non-inverse input and a triangular matrix for cholesky decomposition inpiut.
+        """
         super().__init__(num_freqs, frame_size, lookahead=lookahead)
-        self.eps = eps
-        self.normalize_ifc = normalize_ifc
         self.cholesky_decomp = cholesky_decomp
+        self.inverse = inverse
+        self.enforce_constraints = enforce_constraints
+        self.triu_idcs = torch.triu_indices(self.frame_size, self.frame_size, 1)
+        self.tril_idcs = torch.empty_like(self.triu_idcs)
+        self.tril_idcs[0] = self.triu_idcs[1]
+        self.tril_idcs[1] = self.triu_idcs[0]
+        self.eps = eps
+        self.dload = dload
+
+    def get_r_factor(self):
+        """Return an factor f such that Rnn/f in range [-1, 1]"""
+        if self.inverse and self.cholesky_decomp:
+            return 2e4
+        elif self.inverse and not self.cholesky_decomp:
+            return 3e8
+        elif not self.inverse and self.cholesky_decomp:
+            return 5e-5
+        elif not self.inverse and not self.cholesky_decomp:
+            return 1e-6
 
     def forward(self, spec: Tensor, ifc: Tensor, iRnn: Tensor) -> Tensor:
         """Multi-frame MVDR filter based on Rnn**-1 and speech IFC vector.
@@ -282,7 +372,7 @@ class MfMvdr(MultiFrameModule):
         Args:
             spec (complex Tensor): Spectrogram of shape [B, C, T, F]
             ifc (complex Tensor): Inter-frame speech correlation vector [B, C*N*2, T, F]
-            iRnn (complex Tensor): Inverse noise covariance matrix Rnn**-1 [B, T, F (N**2)*2] OR
+            iRnn (complex Tensor): (Inverse) noise covariance matrix Rnn**-1 [B, T, F (N**2)*2] OR
                 cholesky_decomp Rnn=L*L^H of the same shape.
 
         Returns:
@@ -291,15 +381,31 @@ class MfMvdr(MultiFrameModule):
         spec_u = self.spec_unfold(torch.view_as_complex(spec))
         iRnn = torch.view_as_complex(iRnn.unflatten(3, (self.frame_size, self.frame_size, 2)))
         if self.cholesky_decomp:
-            iRnn = iRnn * iRnn.transpose(3, 4).conj()
+            if self.enforce_constraints:
+                # Upper triangular (wo. diagonal) must be zero
+                iRnn[:, :, :, self.triu_idcs[0], self.triu_idcs[1]] = 0.0
+            # Revert cholesky decomposition
+            iRnn = iRnn.matmul(iRnn.transpose(3, 4).conj())
+        if self.enforce_constraints and not self.inverse and not self.cholesky_decomp:
+            # If we have a cholesky_decomp input the constraints are already enforced.
+            # We have a standard correlation matrix as input. Imaginary part on the diagonal should be 0.
+            torch.diagonal(iRnn, dim1=-1, dim2=-2).imag = 0.0
+            # Triu should be complex conj of tril
+            tril_conj = iRnn[:, :, :, self.tril_idcs[0], self.tril_idcs[1]].conj()
+            iRnn[:, :, :, self.triu_idcs[0], self.triu_idcs[1]] = tril_conj
         ifc = torch.view_as_complex(ifc.unflatten(3, (self.frame_size, 2)))
-        if self.normalize_ifc:
-            ifc0 = ifc[..., -1]
-            ifc = ifc / (ifc0.unsqueeze(-1) + self.eps)
         spec_f = spec_u.narrow(-2, 0, self.num_freqs)
-        numerator = torch.einsum("...nm,...m->...n", iRnn, ifc)  # [B, C, F, N]
+        if not self.inverse:  # Is already an inverse input. No need to inverse it again.
+            # Regularization on diag for stability
+            iRnn = _tik_reg(iRnn, self.dload, self.eps)
+            # Compute weights by solving the equation system
+            numerator = torch.linalg.solve(iRnn, ifc)
+        else:  # We already have an inverse estimate. Just compute the linear combination.
+            numerator = torch.einsum("...nm,...m->...n", iRnn, ifc)  # [B, C, F, N]
         denumerator = torch.einsum("...n,...n->...", ifc.conj(), numerator)
-        w = (numerator / (denumerator.real.unsqueeze(-1) + self.eps)).unsqueeze(1)
+        # Normalize numerator
+        scale = ifc[..., -1, None].conj()
+        w = (numerator * scale / (denumerator.real.unsqueeze(-1) + self.eps)).unsqueeze(1)
         spec_f = self.apply_coefs(spec_f, w)
         if self.training:
             spec = spec.clone()
@@ -308,7 +414,7 @@ class MfMvdr(MultiFrameModule):
 
 
 # From torchaudio
-def _compute_mat_trace(input: torch.Tensor, dim1: int = -1, dim2: int = -2) -> torch.Tensor:
+def _compute_mat_trace(input: torch.Tensor, dim1: int = -2, dim2: int = -1) -> torch.Tensor:
     r"""Compute the trace of a Tensor along ``dim1`` and ``dim2`` dimensions.
     Args:
         input (torch.Tensor): Tensor of dimension `(..., channel, channel)`
@@ -346,28 +452,41 @@ def _tik_reg(mat: torch.Tensor, reg: float = 1e-7, eps: float = 1e-8) -> torch.T
     return mat
 
 
-def compute_cov(X: Tensor, N: int):
+def compute_corr(X: Tensor, N: int):
     Xw = F.pad(X, (0, 0, N - 1, 0)).unfold(1, N, 1)
     Rxx = torch.einsum("...n,...m->...mn", Xw, Xw.conj())
     return Rxx
 
 
-def compute_ideal_wf():
-    from icecream import install
+def compute_ideal_wf(
+    rxx_via_rssrnn=True, cholesky_decomp=False, inverse=True, enforce_constraints=True, manual=False
+):
+    from icecream import ic, install
 
     import libdf
     from df.config import config
     from df.io import load_audio, save_audio
     from df.model import ModelParams
 
+    torch.set_printoptions(linewidth=140)
+
     ORDER = 5
-    DLOAD = 1e-9
+    DLOAD = 1e-7
     EPS = 1e-8
+    if cholesky_decomp and inverse:
+        DLOAD = 1e-4
+        EPS = 1e-5
 
     install()
+    ic.includeContext = True
 
-    config.use_defaults()
+    config.use_defaults(allow_reload=True)
     p = ModelParams()
+    p.fft_size = 96
+    p.hop_size = 24
+    p.sr = 24000
+    n_freqs = p.fft_size // 2 + 1
+
     df = libdf.DF(sr=p.sr, fft_size=p.fft_size, hop_size=p.hop_size, nb_bands=p.nb_erb)
     s = load_audio("assets/clean_freesound_33711.wav", p.sr, num_frames=5 * p.sr)[0].mean(
         0, keepdim=True
@@ -378,31 +497,49 @@ def compute_ideal_wf():
     x = s + n
     save_audio("out/noisy.wav", x, p.sr)
 
-    wf = MfWf(p.fft_size // 2 + 1, ORDER)
+    wf = MfWf(
+        n_freqs,
+        ORDER,
+        cholesky_decomp=cholesky_decomp,
+        inverse=inverse,
+        enforce_constraints=enforce_constraints,
+    )
 
     X, S, N = [torch.from_numpy(df.analysis(x.numpy())) for x in (x, s, n)]
     Xw = F.pad(X, (0, 0, ORDER - 1, 0)).unfold(1, ORDER, 1)
-    Rss, Rnn = [compute_cov(X, ORDER) for X in (S, N)]
+    Rss, Rnn, Rxx = [compute_corr(A, ORDER) for A in (S, N, X)]
     ifc = Rss[..., -1]
     Rnn = _tik_reg(Rnn, DLOAD, EPS)
-    Rxx = Rss + Rnn  # Adding these is a lot better compared to estimating Rxx from X
-    R_inv = torch.inverse(Rxx)
+    if rxx_via_rssrnn:
+        # Adding these is slightly better compared to estimating Rxx from X
+        Rxx = Rss + Rnn
+    else:
+        Rxx = _tik_reg(Rxx, DLOAD, EPS)
+    A = Rxx
+    if inverse:
+        A = torch.linalg.inv(A)
+    if cholesky_decomp:
+        A, info = torch.linalg.cholesky_ex(A)
+        print("Number of errors during cholesky_decomp:", torch.where(info > 0, 1, 0).sum())
+    ic(A.abs().mean())
     # Manual way
-    w = torch.einsum("...nm,...m->...n", R_inv, ifc)
-    Y = torch.einsum("...fn,...fn->...f", Xw, w)
-    # Using torch module (which expects real valued flattened input)
-    Y = torch.view_as_complex(
-        wf(
-            torch.view_as_real(X).unsqueeze(1),
-            torch.view_as_real(ifc).flatten(3),
-            torch.view_as_real(R_inv).flatten(3),
-        ).squeeze(1)
-    )
+    if manual:
+        w = torch.einsum("...nm,...m->...n", A, ifc)
+        Y = torch.einsum("...fn,...fn->...f", Xw, w)
+    else:
+        # Using torch module (which expects real valued flattened input)
+        Y = torch.view_as_complex(
+            wf(
+                torch.view_as_real(X).unsqueeze(1),
+                torch.view_as_real(ifc).flatten(3),
+                torch.view_as_real(A).flatten(3),
+            ).squeeze(1)
+        )
     y = df.synthesis(Y.numpy())
-    save_audio("out/ideal_mfwf.wav", y, p.sr)
+    save_audio("out/ideal_mfwf_c{}_i{}.wav".format(int(cholesky_decomp), int(inverse)), y, p.sr)
 
 
-def compute_ideal_mvdr(cholesky_decomp=False):
+def compute_ideal_mvdr(cholesky_decomp=False, inverse=True, enforce_constraints=True, manual=False):
     from icecream import ic, install
 
     import libdf
@@ -411,15 +548,25 @@ def compute_ideal_mvdr(cholesky_decomp=False):
     from df.model import ModelParams
 
     ic.includeContext = True
+    torch.set_printoptions(linewidth=120)
 
     ORDER = 5
-    DLOAD = 1e-9
-    EPS = 1e-9
+    DLOAD = 1e-7
+    EPS = 1e-8
+    if cholesky_decomp and inverse:
+        DLOAD = 1e-4
+        EPS = 1e-5
 
     install()
 
-    config.use_defaults()
+    config.use_defaults(allow_reload=True)
     p = ModelParams()
+    p.fft_size = 96
+    p.hop_size = 24
+    p.sr = 24000
+    n_freqs = p.fft_size // 2 + 1
+    n_freqs = p.fft_size // 2 + 1
+
     df = libdf.DF(sr=p.sr, fft_size=p.fft_size, hop_size=p.hop_size, nb_bands=p.nb_erb)
     s = load_audio("assets/clean_freesound_33711.wav", p.sr, num_frames=5 * p.sr)[0].mean(
         0, keepdim=True
@@ -430,41 +577,60 @@ def compute_ideal_mvdr(cholesky_decomp=False):
     x = s + n
     save_audio("out/noisy.wav", x, p.sr)
 
-    mvdr = MfMvdr(p.fft_size // 2 + 1, ORDER, cholesky_decomp=cholesky_decomp)
+    mvdr = MfMvdr(
+        n_freqs,
+        ORDER,
+        cholesky_decomp=cholesky_decomp,
+        inverse=inverse,
+        enforce_constraints=enforce_constraints,
+    )
 
     X, S, N = [torch.from_numpy(df.analysis(x.numpy())) for x in (x, s, n)]
     Xw = F.pad(X, (0, 0, ORDER - 1, 0)).unfold(1, ORDER, 1)
-    Rss, Rnn = [compute_cov(x, ORDER) for x in (S, N)]
+    Rss, Rnn = [compute_corr(x, ORDER) for x in (S, N)]
 
-    # A: Normalized IFC
+    # A: IFC, needs to be normalized later
     ifc = Rss[..., -1]
-    ifc0 = ifc[..., -1]
-    ifc0[:, 0] = 1
-    ifc = ifc / (ifc0.unsqueeze(-1) + EPS)
 
     # B: IFC via EVD
     _, v = torch.linalg.eigh(Rss)
     ifc = v[..., -1]  # Choose highest eigenvector
 
-    Rnn = _tik_reg(Rnn, DLOAD, EPS)
-    R_inv = torch.inverse(Rnn)
+    A = _tik_reg(Rnn, DLOAD, EPS)
+    if inverse:
+        A = torch.linalg.inv(A)
     if cholesky_decomp:
-        L, info = torch.linalg.cholesky_ex(R_inv)
-        ic(torch.where(info > 0, 1, 0).sum())
-        R_inv = L
+        A, info = torch.linalg.cholesky_ex(A)
+        print("Number of errors during cholesky_decomp:", torch.where(info > 0, 1, 0).sum())
+    ic(A.abs().mean())
     # Manual way
-    num = torch.linalg.solve(Rnn, ifc)
-    # num = torch.einsum("...nm,...m->...n", R_inv, ifc)
-    denum = torch.einsum("...n,...n->...", ifc.conj(), num)
-    w = num / (denum.unsqueeze(-1) + EPS)
-    Y = torch.einsum("...fn,...fn->...f", Xw, w)
+    if manual:
+        ifc0 = ifc[..., -1]
+        ifc = ifc / (ifc0.unsqueeze(-1) + EPS)
+        if cholesky_decomp:
+            A = A.matmul(A.conj().transpose(-1, -2))
+        if inverse:
+            num = torch.einsum("...nm,...m->...n", A, ifc)
+        else:
+            num = torch.linalg.solve(Rnn, ifc)
+        denum = torch.einsum("...n,...n->...", ifc.conj(), num)
+        w = num / (denum.unsqueeze(-1) + EPS)
+        Y = torch.einsum("...fn,...fn->...f", Xw, w)
     # Using torch module (which expects real valued flattened input)
-    Y = torch.view_as_complex(
-        mvdr(
-            torch.view_as_real(X).unsqueeze(1),
-            torch.view_as_real(ifc).flatten(3),
-            torch.view_as_real(R_inv).flatten(3),
-        ).squeeze(1)
-    )
+    else:
+        Y = torch.view_as_complex(
+            mvdr(
+                torch.view_as_real(X).unsqueeze(1),
+                torch.view_as_real(ifc).flatten(3),
+                torch.view_as_real(A).flatten(3),
+            ).squeeze(1)
+        )
     y = df.synthesis(Y.numpy())
-    save_audio("out/ideal_mfmvdr.wav", y, p.sr)
+    save_audio("out/ideal_mfmvdr_c{}_i{}.wav".format(int(cholesky_decomp), int(inverse)), y, p.sr)
+
+
+def compute_all_mf(enforce_constraints=True):
+    for m in (compute_ideal_wf, compute_ideal_mvdr):
+        for c in (True, False):
+            for i in (True, False):
+                m(cholesky_decomp=c, inverse=i, enforce_constraints=enforce_constraints)
