@@ -18,7 +18,7 @@ from torch.types import Number
 from df.checkpoint import check_patience, load_model, read_cp, write_cp
 from df.config import Csv, config
 from df.logger import init_logger, log_metrics, log_model_summary
-from df.loss import Istft, Loss, MaskLoss
+from df.loss import Istft, Loss
 from df.lr import cosine_scheduler
 from df.model import ModelParams
 from df.modules import get_device
@@ -143,6 +143,7 @@ def main():
         batch_size=bs,
         batch_size_eval=bs_eval,
         num_workers=config("NUM_WORKERS", 4, int, section="train"),
+        pin_memory=get_device().type == "cuda",
         max_len_s=config("MAX_SAMPLE_LEN_S", 5.0, float, section="train"),
         fft_size=p.fft_size,
         hop_size=p.hop_size,
@@ -152,6 +153,7 @@ def main():
         p_reverb=config("p_reverb", 0.2, float, section="distortion"),
         p_bw_ext=config("p_bandwidth_ext", 0.0, float, section="distortion"),
         p_clipping=config("p_clipping", 0.0, float, section="distortion"),
+        p_zeroing=config("p_zeroing", 0.0, float, section="distortion"),
         p_air_absorption=config("p_air_absorption", 0.0, float, section="distortion"),
         prefetch=config("NUM_PREFETCH_BATCHES", 32, int, section="train"),
         overfit=overfit,
@@ -160,7 +162,6 @@ def main():
         log_timings=log_timings,
         global_sampling_factor=config("GLOBAL_DS_SAMPLING_F", 1.0, float, section="train"),
         snrs=config("DATALOADER_SNRS", [-5, 0, 5, 10, 20, 40], Csv(int), section="train"),  # type: ignore
-        cache_valid=config("VALIDATION_SET_CACHING", False, bool, section="train"),
         log_level=log_level,
     )
 
@@ -416,7 +417,6 @@ def run_epoch(
                                 batch.snr.detach(),
                                 lsnr.detach().float(),
                                 os.path.join(summary_dir, "nan"),
-                                mask_loss=losses.ml,
                                 prefix=split + f"_e{epoch}_i{i}_b{batch_idx}_ds{clean_idx}",
                                 idx=batch_idx,
                             )
@@ -458,7 +458,6 @@ def run_epoch(
                 batch.snr.detach(),
                 lsnr.detach().float(),
                 summary_dir,
-                mask_loss=losses.ml,
                 prefix=split,
             )
     try:
@@ -502,8 +501,8 @@ def load_opt(
     else:
         params = model.parameters()
     supported = {
-        "adam": lambda p: optim.Adam(p, lr=lr, weight_decay=decay, betas=betas),
-        "adamw": lambda p: optim.AdamW(p, lr=lr, weight_decay=decay, betas=betas),
+        "adam": lambda p: optim.Adam(p, lr=lr, weight_decay=decay, betas=betas, amsgrad=True),
+        "adamw": lambda p: optim.AdamW(p, lr=lr, weight_decay=decay, betas=betas, amsgrad=True),
         "sgd": lambda p: optim.SGD(p, lr=lr, momentum=momentum, nesterov=True, weight_decay=decay),
         "rmsprop": lambda p: optim.RMSprop(p, lr=lr, momentum=momentum, weight_decay=decay),
     }
@@ -572,7 +571,6 @@ def summary_write(
     snrs: Tensor,
     lsnr: Tensor,
     summary_dir: str,
-    mask_loss: Optional[MaskLoss] = None,
     prefix="train",
     idx: Optional[int] = None,
 ):
@@ -588,12 +586,6 @@ def summary_write(
     def synthesis(x: Tensor) -> Tensor:
         return torch.as_tensor(state.synthesis(make_np(as_complex(x.detach()))))
 
-    if mask_loss is not None:
-        ideal = mask_loss.erb_mask_compr(clean[idx], noisy[idx], compressed=False)
-        ideal = noisy[idx] * mask_loss.erb_inv(ideal)
-        torchaudio.save(
-            os.path.join(summary_dir, f"{prefix}_idealmask_snr{snr}.wav"), synthesis(ideal), p.sr
-        )
     torchaudio.save(
         os.path.join(summary_dir, f"{prefix}_clean_snr{snr}.wav"), synthesis(clean[idx]), p.sr
     )
